@@ -3,10 +3,12 @@
 Module for the various widgets that keep an overall view of the status or progress
 of the current run/setup
 """
-
-import gtk, gobject, pango, guiutils, os, logging
+from gi.repository import Gtk, GObject, Pango, GdkPixbuf
+from . import guiutils
+import os
+import logging
 from texttestlib import plugins
-from ordereddict import OrderedDict
+from collections import OrderedDict
 from copy import copy
 
 #
@@ -14,6 +16,8 @@ from copy import copy
 # It is also responsible for keeping the throbber rotating
 # while actions are under way.
 #
+
+
 class StatusMonitorGUI(guiutils.SubGUI):
     def __init__(self, initialMessage):
         guiutils.SubGUI.__init__(self)
@@ -26,23 +30,23 @@ class StatusMonitorGUI(guiutils.SubGUI):
 
     def getWidgetName(self):
         return "_Status bar"
-    
+
     def notifyActionStart(self, lock=True):
         if self.throbber:
-            if self.pixbuf: # pragma: no cover : Only occurs if some code forgot to do ActionStop ...
+            if self.pixbuf:  # pragma: no cover : Only occurs if some code forgot to do ActionStop ...
                 self.notifyActionStop(lock)
             self.pixbuf = self.throbber.get_pixbuf()
             self.throbber.set_from_animation(self.animation)
             if lock:
                 self.throbber.grab_add()
-                
+
     def notifyWindowClosed(self, *args):
         self.closing = True
 
     def notifyActionProgress(self, *args):
         if not self.closing:
-            while gtk.events_pending():
-                gtk.main_iteration(False)
+            while Gtk.events_pending():
+                Gtk.main_iteration_do(False)
 
     def notifyActionStop(self, lock=True):
         if self.throbber:
@@ -56,10 +60,10 @@ class StatusMonitorGUI(guiutils.SubGUI):
             self.label.set_markup(plugins.convertForMarkup(message))
 
     def createView(self):
-        hbox = gtk.HBox()
-        self.label = gtk.Label()
+        hbox = Gtk.HBox()
+        self.label = Gtk.Label()
         self.label.set_name("GUI status")
-        self.label.set_ellipsize(pango.ELLIPSIZE_END)
+        self.label.set_ellipsize(Pango.EllipsizeMode.END)
         # It seems difficult to say 'ellipsize when you'd otherwise need
         # to enlarge the window', so we'll have to settle for a fixed number
         # of max char's ... The current setting (90) is just a good choice
@@ -69,22 +73,21 @@ class StatusMonitorGUI(guiutils.SubGUI):
         self.label.set_max_width_chars(90)
         self.label.set_use_markup(True)
         self.label.set_markup(self.initialMessage)
-        hbox.pack_start(self.label, expand=False, fill=False)
+        hbox.pack_start(self.label, False, False, 0)
         imageDir = plugins.installationDir("images")
         try:
             staticIcon = os.path.join(imageDir, "throbber_inactive.png")
-            temp = gtk.gdk.pixbuf_new_from_file(staticIcon)
-            self.throbber = gtk.Image()
-            self.throbber.set_from_pixbuf(temp)
+            temp = GdkPixbuf.Pixbuf.new_from_file(staticIcon)
+            self.throbber = Gtk.Image.new_from_pixbuf(temp)
             animationIcon = os.path.join(imageDir, "throbber_active.gif")
-            self.animation = gtk.gdk.PixbufAnimation(animationIcon)
-            hbox.pack_end(self.throbber, expand=False, fill=False)
-        except Exception, e:
-            plugins.printWarning("Failed to create icons for the status throbber:\n" + str(e) + \
+            self.animation = GdkPixbuf.PixbufAnimation.new_from_file(animationIcon)
+            hbox.pack_end(self.throbber, False, False, 0)
+        except Exception as e:
+            plugins.printWarning("Failed to create icons for the status throbber:\n" + str(e) +
                                  "\nAs a result, the throbber will be disabled.", stdout=True)
             self.throbber = None
-        self.widget = gtk.Frame()
-        self.widget.set_shadow_type(gtk.SHADOW_ETCHED_IN)
+        self.widget = Gtk.Frame()
+        self.widget.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
         self.widget.add(hbox)
         self.widget.show_all()
         return self.widget
@@ -101,9 +104,10 @@ class ProgressBarGUI(guiutils.SubGUI):
 
     def shouldShow(self):
         return self.dynamic
-    
+
     def createView(self):
-        self.widget = gtk.ProgressBar()
+        self.widget = Gtk.ProgressBar()
+        self.widget.set_show_text(True)
         self.resetBar()
         self.widget.show()
         return self.widget
@@ -114,7 +118,7 @@ class ProgressBarGUI(guiutils.SubGUI):
             if self.addedCount > self.totalNofTests:
                 self.totalNofTests += 1
                 self.resetBar()
-                
+
     def notifyAllRead(self, *args):
         # The initial number was told be the static GUI, treat it as a guess
         # Can be wrong in case versions are defined by testsuite files.
@@ -122,8 +126,8 @@ class ProgressBarGUI(guiutils.SubGUI):
             self.totalNofTests = self.addedCount
             self.resetBar()
 
-    def notifyLifecycleChange(self, dummyTest, dummyState, changeDesc):
-        if changeDesc == "complete":
+    def notifyLifecycleChange(self, test, dummyState, changeDesc):
+        if changeDesc == "complete" and test.classId() == "test-case":
             self.nofCompletedTests += 1
             self.resetBar()
 
@@ -131,7 +135,7 @@ class ProgressBarGUI(guiutils.SubGUI):
         if self.totalNofTests > 0:
             return float(self.nofCompletedTests) / float(self.totalNofTests)
         else:
-            return 0 # No tests yet, haven't read them in
+            return 0  # No tests yet, haven't read them in
 
     def resetBar(self):
         if self.widget:
@@ -145,6 +149,7 @@ class ProgressBarGUI(guiutils.SubGUI):
         else:
             return str(self.nofCompletedTests) + " of " + str(self.totalNofTests) + " tests completed"
 
+
 class ClassificationTree(OrderedDict):
     def addClassification(self, path):
         prevElementName = None
@@ -155,7 +160,7 @@ class ClassificationTree(OrderedDict):
             else:
                 elementName = element
                 elementTuple = element, None
-            if not self.has_key(elementName):
+            if elementName not in self:
                 self[elementName] = []
             if prevElementName and elementTuple not in self[prevElementName]:
                 self[prevElementName].append(elementTuple)
@@ -163,15 +168,17 @@ class ClassificationTree(OrderedDict):
 
 # Class that keeps track of (and possibly shows) the progress of
 # pending/running/completed tests
+
+
 class TestProgressMonitor(guiutils.SubGUI):
     def __init__(self, dynamic, testCount):
         guiutils.SubGUI.__init__(self)
-        self.classifications = {} # map from test to list of iterators where it exists
+        self.classifications = {}  # map from test to list of iterators where it exists
 
         # Each row has 'type', 'number', 'show', 'tests'
-        self.treeModel = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_INT, gobject.TYPE_BOOLEAN, 
-                                       gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_PYOBJECT,
-                                       gobject.TYPE_STRING)
+        self.treeModel = Gtk.TreeStore(GObject.TYPE_STRING, GObject.TYPE_INT, GObject.TYPE_BOOLEAN,
+                                       GObject.TYPE_STRING, GObject.TYPE_STRING, GObject.TYPE_PYOBJECT,
+                                       GObject.TYPE_STRING)
         self.diag = logging.getLogger("Progress Monitor")
         self.progressReport = None
         self.treeView = None
@@ -184,7 +191,8 @@ class TestProgressMonitor(guiutils.SubGUI):
             # using differnt diff tools are run together. However, this isn't very likely and we prefer not
             # to recalculate all the time...
             diffTool = guiutils.guiConfig.getValue("text_diff_program")
-            self.diffFilterGroup = plugins.TextTriggerGroup(guiutils.guiConfig.getCompositeValue("text_diff_program_filters", diffTool))
+            self.diffFilterGroup = plugins.TextTriggerGroup(
+                guiutils.guiConfig.getCompositeValue("text_diff_program_filters", diffTool))
             self.maxLengthForGrouping = guiutils.guiConfig.getValue("lines_of_text_difference")
             if testCount > 0:
                 colour = guiutils.guiConfig.getTestColour("not_started")
@@ -198,37 +206,38 @@ class TestProgressMonitor(guiutils.SubGUI):
         return self.dynamic
 
     def createView(self):
-        self.treeView = gtk.TreeView(self.treeModel)
+        self.treeView = Gtk.TreeView(self.treeModel)
         self.treeView.set_name("Test Status View")
         self.treeView.set_enable_search(False)
         selection = self.treeView.get_selection()
-        selection.set_mode(gtk.SELECTION_MULTIPLE)
-        selection.set_select_function(self.canSelect)
+        selection.set_mode(Gtk.SelectionMode.MULTIPLE)
+        selection.set_select_function(self.canSelect, None)
         selection.connect("changed", self.selectionChanged)
-        textRenderer = gtk.CellRendererText()
+        textRenderer = Gtk.CellRendererText()
         textRenderer.set_property('wrap-width', 350)
-        textRenderer.set_property('wrap-mode', pango.WRAP_WORD_CHAR)
-        numberRenderer = gtk.CellRendererText()
+        textRenderer.set_property('wrap-mode', Pango.WrapMode.WORD_CHAR)
+        numberRenderer = Gtk.CellRendererText()
         numberRenderer.set_property('xalign', 1)
-        statusColumn = gtk.TreeViewColumn("Status", textRenderer, text=0, background=3, font=4)
-        numberColumn = gtk.TreeViewColumn("Number", numberRenderer, text=1, background=3, font=4)
+        statusColumn = Gtk.TreeViewColumn("Status", textRenderer, text=0, background=3, font=4)
+        numberColumn = Gtk.TreeViewColumn("Number", numberRenderer, text=1, background=3, font=4)
         statusColumn.set_resizable(True)
         numberColumn.set_resizable(True)
         self.treeView.append_column(statusColumn)
         self.treeView.append_column(numberColumn)
-        toggle = gtk.CellRendererToggle()
+        toggle = Gtk.CellRendererToggle()
         toggle.set_property('activatable', True)
         toggle.connect("toggled", self.showToggled)
-        toggleColumn = gtk.TreeViewColumn("Visible", toggle, active=2)
+        toggleColumn = Gtk.TreeViewColumn("Visible", toggle, active=2)
         toggleColumn.set_resizable(True)
         toggleColumn.set_alignment(0.5)
         self.treeView.append_column(toggleColumn)
         self.treeView.show()
-        return self.addScrollBars(self.treeView, hpolicy=gtk.POLICY_NEVER)
+        return self.addScrollBars(self.treeView, hpolicy=Gtk.PolicyType.NEVER)
 
-    def canSelect(self, path):
-        pathIter = self.treeModel.get_iter(path)
-        return self.treeModel.get_value(pathIter, 2)
+    @staticmethod
+    def canSelect(selection, model, path, is_selected, user_data):
+        pathIter = model.get_iter(path)
+        return model.get_value(pathIter, 2)
 
     def notifyAdd(self, test, initial):
         if self.dynamic and test.classId() == "test-case":
@@ -239,7 +248,7 @@ class TestProgressMonitor(guiutils.SubGUI):
         # Fix the not started count in case the initial guess was wrong
         if self.testCount > 0:
             self.diag.info("Reading complete, updating not-started count to actual answer")
-            iter = self.treeModel.get_iter_root()
+            iter = self.treeModel.get_iter_first()
             actualTestCount = len(self.treeModel.get_value(iter, 5))
             measuredTestCount = self.treeModel.get_value(iter, 1)
             if actualTestCount != measuredTestCount:
@@ -247,11 +256,12 @@ class TestProgressMonitor(guiutils.SubGUI):
 
     def selectionChanged(self, selection):
         if hasattr(selection, "unseen_changes"):
-            return # hack in StoryText so we can reproduce manual behaviour
+            return  # hack in StoryText so we can reproduce manual behaviour
 
         # For each selected row, select the corresponding rows in the test treeview
         # and the corresponding files in the test fileview
         tests, fileStems = [], []
+
         def addSelected(treemodel, dummyPath, iter, *args):
             for test in treemodel.get_value(iter, 5):
                 if test not in tests:
@@ -279,7 +289,7 @@ class TestProgressMonitor(guiutils.SubGUI):
         for line in diff.splitlines():
             if self.diffFilterGroup.stringContainsText(line):
                 filteredDiff += line + "\n"
-        return filteredDiff or diff # If we filter it all away, should assume it was a new file and return it unchanged
+        return filteredDiff or diff  # If we filter it all away, should assume it was a new file and return it unchanged
 
     def getDifferenceType(self, fileComp):
         if fileComp.missingResult():
@@ -289,108 +299,150 @@ class TestProgressMonitor(guiutils.SubGUI):
         else:
             return "Differences"
 
-    def getClassifiers(self, test, state, changeDesc):
-        classifiers = ClassificationTree()
+    def getCategoryDescriptionClassifier(self, state, changeDesc):
         catDesc = self.getCategoryDescription(state)
         if state.isMarked():
-            if state.briefText == catDesc:
-                # Just in case - otherwise we get an infinite loop...
-                classifiers.addClassification([ catDesc, "Marked as Marked" ])
-            else:
-                classifiers.addClassification([ catDesc, state.briefText ])
-            return classifiers
+            briefText = "Marked as Marked" if state.briefText == catDesc else state.briefText
+            return [catDesc, briefText]
 
         if not state.isComplete() or not state.hasFailed():
-            classification = [ catDesc ]
+            classification = [catDesc]
             if "approve" in changeDesc:
                 classification.append("Approved")
             elif not state.isComplete() and state.briefText:
                 subState = state.briefText.split()[0]
                 if subState != state.defaultBriefText:
                     classification.append(subState)
-            classifiers.addClassification(classification)
-            return classifiers
+            return classification
 
-        if not state.isSaveable() or state.warnOnSave(): # If it's not saveable, don't classify it by the files
+        if not state.isSaveable() or state.warnOnSave():  # If it's not saveable, don't classify it by the files
             details = state.getTypeBreakdown()[1]
             self.diag.info("Adding unsaveable : " + catDesc + " " + details)
-            classifiers.addClassification([ "Failed", catDesc, details ])
-            return classifiers
-
-        comparisons = state.getComparisons()
-        for fileComp in filter(lambda c: c.getType() == "failure", comparisons):
-            summary = self.getFileSummary(fileComp)
-            fileClass = [ "Failed", self.getDifferenceType(fileComp), (summary, fileComp.stem) ]
-
-            filteredDiff = self.getFilteredDiff(fileComp)
-            if filteredDiff is not None:
-                groupNames, summaryDiffs = self.diffStore.setdefault(summary, ({}, OrderedDict()))
-                testList, groupName = summaryDiffs.setdefault(filteredDiff, ([], None))
-                if test not in testList:
-                    testList.append(test)
-                if len(testList) > 1 and groupName is None:
-                    groupName = self.setGroupName(groupNames, summaryDiffs, filteredDiff)
-                if groupName:
-                    fileClass.append(("Group " + groupName, fileComp.stem))
-
-            self.diag.info("Adding file classification for " + repr(fileComp) + " = " + repr(fileClass))
-            classifiers.addClassification(fileClass)
-
-        for fileComp in filter(lambda c: c.getType() != "failure", comparisons):
-            summary = fileComp.getSummary(includeNumbers=False)
-            desc = self.getCategoryDescription(state, summary)
-            stem = fileComp.stem
-            fileClass = [ "Failed", "Performance differences", (desc, stem) ]
-            toleranceRange = fileComp.getToleranceMultipleRange(test)
-            if toleranceRange:
-                fileClass.append((toleranceRange, stem))
-            self.diag.info("Adding file classification for " + repr(fileComp) + " = " + repr(fileClass))
-            classifiers.addClassification(fileClass)
+            return ["Failed", catDesc, details]
 
+    def getClassifiers(self, test, state, changeDesc):
+        classifiers = ClassificationTree()
+        catDescClassifier = self.getCategoryDescriptionClassifier(state, changeDesc)
+        if catDescClassifier:
+            classifiers.addClassification(catDescClassifier)
+        else:
+            self.addComparisonClassifiers(classifiers, test, state)
         return classifiers
 
+    def addComparisonClassifiers(self, classifiers, test, state):
+        failureComparisons, perfComparisons = [], []
+        for c in state.getComparisons():
+            l = failureComparisons if c.getType() == "failure" else perfComparisons
+            l.append(c)
+
+        for fileComp in failureComparisons:
+            self.addFailureComparisonClassifiers(classifiers, test, state, fileComp)
+
+        for fileComp in perfComparisons:
+            fileClass = self.getPerformanceComparisonClassifier(test, state, fileComp)
+            classifiers.addClassification(fileClass)
+
+    def addFailureComparisonClassifiers(self, classifiers, test, state, fileComp):
+        summary = self.getFileSummary(fileComp)
+        fileClass = ["Failed", self.getDifferenceType(fileComp), (summary, fileComp.stem)]
+
+        filteredDiff = self.getFilteredDiff(fileComp)
+        extraGroupName = None
+        if filteredDiff is not None:
+            groupNames, summaryDiffs, ungrouped = self.diffStore.setdefault(summary, ({}, OrderedDict(), []))
+            hasGroups = len(groupNames) > 0
+            testList, groupName = summaryDiffs.setdefault(filteredDiff, ([], None))
+            if test not in testList:
+                testList.append(test)
+            if groupName is None:
+                onlyIfRepeated = len(testList) == 1
+                groupName, extraGroupName = self.setGroupName(groupNames, summaryDiffs, filteredDiff, onlyIfRepeated)
+            if groupName:
+                fileClass.append(("Group " + groupName, fileComp.stem))
+            else:
+                ungrouped.append(test)
+                if groupNames:
+                    fileClass.append(("Ungrouped", fileComp.stem))
+
+        if extraGroupName:
+            extraFileClass = copy(fileClass[:-1])
+            extraFileClass.append(("Group " + extraGroupName, fileComp.stem))
+            self.diag.info("Adding extra file classification for " + repr(fileComp) + " = " + repr(extraFileClass))
+            classifiers.addClassification(extraFileClass)
+        self.diag.info("Adding file classification for " + repr(fileComp) + " = " + repr(fileClass))
+        classifiers.addClassification(fileClass)
+        if filteredDiff is not None and groupName and not hasGroups:
+            for _ in ungrouped:
+                extraFileClass = copy(fileClass[:-1])
+                extraFileClass.append(("Ungrouped", fileComp.stem))
+                self.diag.info("Adding extra ungrouped category for " + repr(fileComp) + " = " + repr(extraFileClass))
+                classifiers.addClassification(extraFileClass)
+
+    def getPerformanceComparisonClassifier(self, test, state, fileComp):
+        summary = fileComp.getSummary(includeNumbers=False)
+        desc = self.getCategoryDescription(state, summary)
+        stem = fileComp.stem
+        fileClass = ["Failed", "Performance differences", (desc, stem)]
+        toleranceRange = fileComp.getToleranceMultipleRange(test)
+        if toleranceRange:
+            fileClass.append((toleranceRange, stem))
+        self.diag.info("Adding file classification for " + repr(fileComp) + " = " + repr(fileClass))
+        return fileClass
+
     def extractRepeats(self, filteredDiff):
         size = len(filteredDiff)
-        smallPrimes = [ 2, 3, 5, 7, 11, 13, 17, 19 ] # Surely we won't repeat stuff more than 20 times :)
+        smallPrimes = [2, 3, 5, 7, 11, 13, 17, 19]  # Surely we won't repeat stuff more than 20 times :)
         for prime in smallPrimes:
             if size > prime and size % prime == 0:
-                chunkSize = size / prime
+                chunkSize = size // prime
                 firstPart = filteredDiff[:chunkSize]
                 if filteredDiff == firstPart * prime:
                     return firstPart, prime
         return None, None
 
-    def setGroupName(self, groupNames, summaryDiffs, filteredDiff):
-        groupName = self.getGroupName(groupNames, summaryDiffs, filteredDiff)
+    def setGroupName(self, groupNames, summaryDiffs, filteredDiff, onlyIfRepeated):
+        groupName, extraGroupName = self.getGroupName(groupNames, summaryDiffs, filteredDiff, onlyIfRepeated)
+        if not groupName:
+            return None, None
+
         groupNames[groupName] = filteredDiff
         tests = summaryDiffs[filteredDiff][0] if filteredDiff in summaryDiffs else []
         summaryDiffs[filteredDiff] = (tests, groupName)
-        return groupName
-                    
-    def getGroupName(self, groupNames, summaryDiffs, filteredDiff):
+        return groupName, extraGroupName
+
+    def getGroupName(self, groupNames, summaryDiffs, filteredDiff, onlyIfRepeated):
         self.diag.info("Getting group name for " + repr(filteredDiff))
         singleVersion, timesRepeated = self.extractRepeats(filteredDiff)
         if singleVersion:
+            self.diag.info("Extracted repeats of " + repr(singleVersion))
             _, group = summaryDiffs.get(singleVersion, (None, None))
             if group is None:
-                group = self.setGroupName(groupNames, summaryDiffs, singleVersion)
+                group, _ = self.setGroupName(groupNames, summaryDiffs, singleVersion, False)
+                self.diag.info("Created group " + repr(group))
+                tests, _ = summaryDiffs.get(singleVersion, (None, None))
+                extraGroupName = group if len(tests) == 1 else None
+            else:
+                extraGroupName = None
+                self.diag.info("Found group " + repr(group))
+
             if "*" in group:
                 core, timeStr = group.split("*")
-                return core + "*" + str(timesRepeated * int(timeStr))
+                return core + "*" + str(timesRepeated * int(timeStr)), None
             else:
-                return str(group) + "*" + str(timesRepeated)
-        else:
+                return str(group) + "*" + str(timesRepeated), extraGroupName
+        elif not onlyIfRepeated:
             group = len(groupNames) + 1
-            return str(group)
-    
+            return str(group), None
+        return None, None
+
     def notifySelectInGroup(self, fileComp):
         summary = self.getFileSummary(fileComp)
-        _, summaryDiffs = self.diffStore.get(summary, {})
+        _, summaryDiffs, _ = self.diffStore.get(summary, {})
         filteredDiff = self.getFilteredDiff(fileComp)
         testList, groupName = summaryDiffs.get(filteredDiff, ([], False))
         if groupName:
             self.notify("SetTestSelection", testList)
-        
+
     def getFileSummary(self, fileComp):
         return fileComp.getSummary(includeNumbers=False)
 
@@ -401,32 +453,38 @@ class TestProgressMonitor(guiutils.SubGUI):
 
     def removeFromModel(self, test):
         for iter in self.findTestIterators(test):
+            self.removeFromIter(iter, test)
+
+    def removeFromIter(self, iter, test):
+        allTests = self.treeModel.get_value(iter, 5)
+        if test in allTests:
             testCount = self.treeModel.get_value(iter, 1)
             self.treeModel.set_value(iter, 1, testCount - 1)
             if testCount == 1:
                 self.treeModel.set_value(iter, 3, "white")
                 self.treeModel.set_value(iter, 4, "")
-            allTests = self.treeModel.get_value(iter, 5)
             allTests.remove(test)
             self.diag.info("Removing test " + repr(test) + " from node " + self.treeModel.get_value(iter, 0))
             self.treeModel.set_value(iter, 5, allTests)
 
     def removeFromDiffStore(self, test):
-        for _, fileInfo in self.diffStore.values():
-            for testList, _ in fileInfo.values():
+        for _, fileInfo, ungrouped in list(self.diffStore.values()):
+            for testList, _ in list(fileInfo.values()):
                 if test in testList:
                     testList.remove(test)
+            if test in ungrouped:
+                ungrouped.remove(test)
 
     def insertTest(self, test, state, changeDesc, incrementCount):
         self.classifications[test] = []
         classifiers = self.getClassifiers(test, state, changeDesc)
-        nodeClassifier = classifiers.keys()[0]
+        nodeClassifier = list(classifiers.keys())[0]
         defaultColour, defaultVisibility = self.getCategorySettings(state.category, nodeClassifier, classifiers)
         return self.addTestForNode(test, defaultColour, defaultVisibility, nodeClassifier, classifiers, incrementCount)
 
     def getCategorySettings(self, category, nodeClassifier, classifiers):
         # Use the category description if there is only one level, otherwise rely on the status names
-        if len(classifiers.get(nodeClassifier)) == 0 or category in [ "failure", "success", "bug" ]:
+        if len(classifiers.get(nodeClassifier)) == 0 or category in ["failure", "success", "bug"]:
             return guiutils.guiConfig.getTestColour(category), guiutils.guiConfig.showCategoryByDefault(category)
         else:
             return None, True
@@ -435,21 +493,33 @@ class TestProgressMonitor(guiutils.SubGUI):
         resultType, summary = state.getTypeBreakdown()
         catDesc = self.getCategoryDescription(state, resultType)
         mainColour = guiutils.guiConfig.getTestColour(catDesc, guiutils.guiConfig.getTestColour(resultType))
-        # Don't change suite states when unmarking tests
-        updateSuccess = state.hasSucceeded() and changeDesc != "unmarked"
-        self.notify("TestAppearance", test, summary, mainColour, colour, updateSuccess, "approve" in changeDesc)
-        self.notify("Visibility", [ test ], self.shouldBeVisible(test))
+        self.notify("TestAppearance", test, summary, mainColour, colour, "approve" in changeDesc)
+        self.notify("Visibility", [test], self.shouldBeVisible(test))
+
+    def removeFromUngroupedNode(self, test, parentIter):
+        self.diag.info("Removing previously ungrouped " + repr(test))
+        ungroupedIter = self.findIter("Ungrouped", parentIter)
+        if ungroupedIter is not None:
+            self.removeFromIter(ungroupedIter, test)
 
     def getInitialTestsForNode(self, test, parentIter, nodeClassifier):
         if nodeClassifier.startswith("Group "):
             groupName = nodeClassifier[6:]
             parentName = self.treeModel.get_value(parentIter, 0)
-            groupNames, summaryDiffs = self.diffStore.get(parentName)
+            groupNames, summaryDiffs, ungrouped = self.diffStore.get(parentName)
             filteredDiff = groupNames.get(groupName)
             if filteredDiff is not None:
                 testList = summaryDiffs[filteredDiff][0]
+                for test in testList:
+                    if test in ungrouped:
+                        self.removeFromUngroupedNode(test, parentIter)
+                        ungrouped.remove(test)
                 return copy(testList)
-        return [ test ]
+        elif nodeClassifier == "Ungrouped":
+            parentName = self.treeModel.get_value(parentIter, 0)
+            _, _, ungrouped = self.diffStore.get(parentName)
+            return copy(ungrouped)
+        return [test]
 
     def addTestForNode(self, test, defaultColour, defaultVisibility, nodeClassifier,
                        classifiers, incrementCount, parentIter=None, fileStem=None):
@@ -461,13 +531,18 @@ class TestProgressMonitor(guiutils.SubGUI):
             self.insertTestAtIter(nodeIter, test, colour, incrementCount)
             self.classifications[test].append(nodeIter)
         else:
-            visibility = guiutils.guiConfig.showCategoryByDefault(nodeClassifier, parentHidden=not defaultVisibility)
+            visibility = guiutils.guiConfig.showCategoryByDefault(nodeClassifier, parentShown=defaultVisibility)
             initialTests = self.getInitialTestsForNode(test, parentIter, nodeClassifier)
-            nodeIter = self.addNewIter(nodeClassifier, parentIter, colour, visibility, len(initialTests), initialTests, fileStem)
-            for initTest in initialTests:
-                self.diag.info("New node " + nodeClassifier + ", colour = " + repr(colour) + ", visible = " + repr(visibility) + " : add " + repr(initTest))
-                self.classifications[initTest].append(nodeIter)
-
+            if len(initialTests):
+                nodeIter = self.addNewIter(nodeClassifier, parentIter, colour, visibility,
+                                           len(initialTests), initialTests, fileStem)
+                for initTest in initialTests:
+                    self.diag.info("New node " + nodeClassifier + ", colour = " + repr(colour) +
+                                   ", visible = " + repr(visibility) + " : add " + repr(initTest))
+                    self.classifications[initTest].append(nodeIter)
+            else:
+                self.diag.info("Not adding new node for " + repr(test) + " for node " +
+                               nodeClassifier + ", no tests in category initially")
         subColours = []
         for subNodeClassifier, fileStem in classifiers[nodeClassifier]:
             subColour = self.addTestForNode(test, colour, visibility, subNodeClassifier,
@@ -488,7 +563,7 @@ class TestProgressMonitor(guiutils.SubGUI):
         if incrementCount:
             self.treeModel.set_value(iter, 1, testCount + 1)
         allTests.append(test)
-        
+
     def addNewIter(self, classifier, parentIter, colour, visibility, testCount, tests=[], fileStem=None):
         modelAttributes = [classifier, testCount, visibility, colour, "bold", tests, fileStem]
         newIter = self.insertIntoModel(parentIter, modelAttributes)
@@ -499,7 +574,8 @@ class TestProgressMonitor(guiutils.SubGUI):
     def insertIntoModel(self, parentIter, modelAttributes):
         if parentIter:
             paddedClassifier = plugins.padNumbersWithZeroes(modelAttributes[0])
-            follower = self.findChildIter(parentIter, lambda name: plugins.padNumbersWithZeroes(name) > paddedClassifier)
+            follower = self.findChildIter(
+                parentIter, lambda name: plugins.padNumbersWithZeroes(name) > paddedClassifier)
             return self.treeModel.insert_before(parentIter, follower, modelAttributes)
         else:
             return self.treeModel.append(parentIter, modelAttributes)
@@ -522,7 +598,7 @@ class TestProgressMonitor(guiutils.SubGUI):
             self.removeFromDiffStore(test)
         colourInserted = self.insertTest(test, state, changeDesc, incrementCount=True)
         self.updateTestAppearance(test, state, changeDesc, colourInserted)
-        
+
     def removeParentIters(self, iters):
         noParents = []
         for iter1 in iters:
@@ -542,13 +618,14 @@ class TestProgressMonitor(guiutils.SubGUI):
         iters = self.findTestIterators(test)
         # ignore the parent nodes where visibility is concerned
         visibilityIters = self.removeParentIters(iters)
-        self.diag.info("Visibility for " + repr(test) + " : iters " + repr(map(self.treeModel.get_path, visibilityIters)))
+        self.diag.info("Visibility for " + repr(test) + " : iters " +
+                       repr(list(map(self.treeModel.get_path, visibilityIters))))
         for nodeIter in visibilityIters:
             visible = self.treeModel.get_value(nodeIter, 2)
             if visible:
                 return True
         return False
-    
+
     def getAllChildIters(self, iter):
         # Toggle all children too
         childIters = []
@@ -578,18 +655,21 @@ class TestProgressMonitor(guiutils.SubGUI):
         self.notify("Visibility", changedTests, newValue)
         if newValue is False:
             selection = self.treeView.get_selection()
-            
-            if selection.path_is_selected(path):
-                #WORKAROUND: selection.unselect_path(path) Doesn't seem to work here
-                selection.set_mode(gtk.SELECTION_SINGLE)
-                selection.set_mode(gtk.SELECTION_MULTIPLE)
 
-    def notifyResetVisibility(self):
+            if selection.path_is_selected(Gtk.TreePath(path)):
+                # WORKAROUND: selection.unselect_path(path) Doesn't seem to work here
+                selection.set_mode(Gtk.SelectionMode.SINGLE)
+                selection.set_mode(Gtk.SelectionMode.MULTIPLE)
+
+    def notifyResetVisibility(self, tests, show):
         self.diag.info("Resetting visibility from current status")
         testsForReset = []
-        self.treeModel.foreach(self.resetNodeVisibility, testsForReset)
-        self.notify("Visibility", testsForReset, True)
 
-    def resetNodeVisibility(self, model, dummyPath, iter, testsForReset):
-        if model.get_value(iter, 2) and not model.iter_has_child(iter):
-            testsForReset += model.get_value(iter, 5)
+        def resetNodeVisibility(model, dummyPath, iter):
+            if model.get_value(iter, 2) == show and not model.iter_has_child(iter):
+                for test in model.get_value(iter, 5):
+                    if test in tests:
+                        testsForReset.append(test)
+
+        self.treeModel.foreach(resetNodeVisibility)
+        self.notify("Visibility", testsForReset, show)
