@@ -3,13 +3,21 @@
 Module containing code that's only run in the slave jobs when running with a grid engine / queue system
 """
 
-import os, sys, time, socket, signal, logging
-from utils import *
+import os
+import sys
+import time
+import socket
+import signal
+import logging
+from .utils import *
 from texttestlib import plugins
 from texttestlib.default.runtest import RunTest
 from texttestlib.default.sandbox import FindExecutionHosts, MachineInfoFinder
 from texttestlib.default.actionrunner import ActionRunner
-from cPickle import dumps
+from texttestlib.utils import getUserName
+from pickle import dumps
+from locale import getpreferredencoding
+
 
 def importAndCallFromQueueSystem(app, *args):
     moduleName = "queuesystem." + queueSystemName(app).lower()
@@ -22,21 +30,25 @@ class RunTestInSlave(RunTest):
         return "RUN (" + ",".join(execMachines) + ")"
 
     def getKillInfoOtherSignal(self, test):
-        if self.killSignal == signal.SIGUSR1:
-            return self.getUserSignalKillInfo(test, "1")
-        elif self.killSignal == signal.SIGUSR2:
-            return self.getUserSignalKillInfo(test, "2")
-        else:
-            return RunTest.getKillInfoOtherSignal(self, test)
-    
+        if os.name == "posix":
+            if self.killSignal == signal.SIGUSR1:
+                return self.getUserSignalKillInfo(test, "1")
+            elif self.killSignal == signal.SIGUSR2:
+                return self.getUserSignalKillInfo(test, "2")
+
+        return RunTest.getKillInfoOtherSignal(self, test)
+
     def getUserSignalKillInfo(self, test, userSignalNumber):
         return importAndCallFromQueueSystem(test.app, "getUserSignalKillInfo", userSignalNumber, self.getExplicitKillInfo)
 
 # Redirect the log mechanism locally in the slave
 # Workaround for NFS slowness, essentially: we can't create these files in the master process in case they
 # don't propagate in time
+
+
 class RedirectLogResponder(plugins.Responder):
     done = False
+
     def notifyAdd(self, test, *args, **kw):
         if not self.done:
             RedirectLogResponder.done = True
@@ -54,60 +66,35 @@ class RedirectLogResponder(plugins.Responder):
                 handler.setFormatter(formatter)
                 plugins.log.addHandler(handler)
 
-class FileTransferResponder(plugins.Responder):
-    def __init__(self, optionMap, *args):
-        plugins.Responder.__init__(self)
-        self.destination = optionMap.get("slavefilesynch")
-        self.transferAll = optionMap.get("keepslave") or optionMap.get("keeptmp")
 
-    def notifyLifecycleChange(self, test, state, changeDesc):
-        if self.destination and changeDesc == "complete" and (self.transferAll or not test.state.hasSucceeded()):
-            plugins.log.info(test.getIndent() + "Transferring files for " + repr(test) + " to " + self.destination)
-            self.copyRemotelyWithRetry(test, test.writeDirectory, "localhost", os.path.dirname(test.writeDirectory), self.destination, ignoreLinks=True)
+class SocketResponder(plugins.Responder, plugins.Observable):
+    synchFiles = False
 
-    def copyRemotelyWithRetry(self, test, *args, **kw):
-        sleepTime = 1
-        errorCode = None
-        for attempt in range(20):
-            errorCode = test.app.copyFileRemotely(*args, **kw)
-            if errorCode:
-                plugins.log.info(test.getIndent() + "File transfer operation failed with error code " + str(errorCode)+ " - waiting " +
-                             str(sleepTime) + " seconds and then trying again.")
-                time.sleep(1)
-            else:
-                if attempt > 0:
-                    plugins.log.info(test.getIndent() + "File transfer operation succeeded on attempt " + str(attempt + 1))
-                return
-        sys.stderr.write("ERROR: File transfer operation failed with error code " + str(errorCode) + "\n")
-        
-    def notifyRequiredTestData(self, test, paths):
-        if self.destination:
-            for path in paths:
-                plugins.log.info(test.getIndent() + "Fetching required test data at " + repr(path) + " from " + self.destination)
-                self.copyRemotelyWithRetry(test, path, self.destination, os.path.dirname(path), "localhost")
-
-
-class SocketResponder(plugins.Responder,plugins.Observable):
     def __init__(self, optionMap, *args):
         plugins.Responder.__init__(self)
         plugins.Observable.__init__(self)
         self.killed = False
+        self.transferAll = optionMap.get("keepslave") or optionMap.get("keeptmp")
         self.testsForRerun = []
         self.serverAddress = self.getServerAddress(optionMap)
+
     def getServerAddress(self, optionMap):
         servAddrStr = optionMap.get("servaddr", os.getenv("CAPTUREMOCK_SERVER"))
         if not servAddrStr:
-            raise plugins.TextTestError, "Cannot run slave, no server address has been provided to send results to!"
+            raise plugins.TextTestError("Cannot run slave, no server address has been provided to send results to!")
         host, port = servAddrStr.split(":")
         return host, int(port)
+
     def connect(self, sendSocket):
-        for i in range(5):
+        attempts = 5
+        for attempt in range(attempts):
             try:
                 sendSocket.connect(self.serverAddress)
                 return True
             except socket.error:
                 time.sleep(1)
-        sys.stderr.write("Failed to connect to " + repr(self.serverAddress) + " : " + self.exceptionOutput())
+                if attempt == attempts -1:
+                    sys.stderr.write("Failed to connect to " + repr(self.serverAddress) + " : " + self.exceptionOutput())
         return False
 
     def exceptionOutput(self):
@@ -118,31 +105,37 @@ class SocketResponder(plugins.Responder,plugins.Observable):
     def notifyKillProcesses(self, *args):
         self.killed = True
 
-    def getProcessIdentifier(self, test):
+    def getProcessIdentifier(self, test, sendFiles):
         identifier = str(os.getpid())
-        if self.killed:
-            identifier += ".NO_REUSE"
-        if test in self.testsForRerun:
+        rerun = test in self.testsForRerun
+        if rerun:
             self.testsForRerun.remove(test)
-            identifier += ".RERUN_TEST"
-        return identifier
+        return makeIdentifierLine(identifier, sendFiles, False, self.killed, rerun)
 
     def notifyRerun(self, test):
         self.testsForRerun.append(test)
 
     def notifyLifecycleChange(self, test, state, changeDesc):
         testData = socketSerialise(test)
-        pickleData = dumps(state)
-        fullData = self.getProcessIdentifier(test) + os.linesep + testData + os.linesep + pickleData
+        protocol = int(os.getenv("TEXTTEST_PICKLE_PROTOCOL", 2)) # Which pickle protocol to use. Useful to set to plain text for self-tests.
+        pickleData = dumps(state, protocol=protocol)
+        sendFiles = self.synchFiles and changeDesc == "complete" and (self.transferAll or not test.state.hasSucceeded())
+        fullData = self.getProcessIdentifier(test, sendFiles) + os.linesep + testData + os.linesep
+        if sendFiles:
+            fullData += directorySerialise(test.writeDirectory) + os.linesep
+        fullDataBytes = fullData.encode(getpreferredencoding()) + pickleData
+        return self.sendAndInterpret(fullDataBytes, self.interpretResponse, state)
+
+    def sendAndInterpret(self, fullData, responseMethod, *args):
         sleepTime = 1
-        for i in range(9):
+        for _ in range(9):
             sendSocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             if not self.connect(sendSocket):
                 return self.notify("NoMoreExtraTests")
             try:
                 response = self.sendData(sendSocket, fullData)
-                return self.interpretResponse(state, response)
-            except socket.error, e:
+                return responseMethod(response, *args) if responseMethod else True
+            except socket.error as e:
                 plugins.log.info("Failed to communicate with master process - waiting " +
                                  str(sleepTime) + " seconds and then trying again.")
                 plugins.log.info("Error received was " + str(e))
@@ -153,15 +146,19 @@ class SocketResponder(plugins.Responder,plugins.Observable):
         sys.stderr.write(message)
         plugins.log.info(message.strip())
         self.notify("NoMoreExtraTests")
-        
+
     def sendData(self, sendSocket, fullData):
         sendSocket.sendall(fullData)
         sendSocket.shutdown(socket.SHUT_WR)
-        response = sendSocket.makefile().read()
+        if self.synchFiles:
+            # Remote socket, possibly firewalls that kill connections, possibly other things. Use timeout and be prepared to retry...
+            # TCP timeout is typically 30 seconds, give up a bit before that
+            sendSocket.settimeout(25)
+        response = sendSocket.makefile('rb').read()
         sendSocket.close()
-        return response
+        return str(response, getpreferredencoding())
 
-    def interpretResponse(self, state, response):
+    def interpretResponse(self, response, state):
         if len(response) > 0:
             appDesc, testPath = socketParse(response)
             appParts = appDesc.split(".")
@@ -169,6 +166,14 @@ class SocketResponder(plugins.Responder,plugins.Observable):
         elif state.isComplete():
             self.notify("NoMoreExtraTests")
 
+    def notifyRequiredTestData(self, test, paths):
+        if self.synchFiles:
+            for path in paths:
+                plugins.log.info(test.getIndent() + "Fetching required test data at " + repr(path) + " ...")
+            data = makeIdentifierLine(str(os.getpid()), getFiles=True) + "\n" + socketSerialise(test) + "\n" + \
+                getUserName() + "@" + getIPAddress([test]) + "\n" + "\n".join(paths)
+            self.sendAndInterpret(data.encode(getpreferredencoding()), None)  # Just wait, no response to interpret
+
 
 class SlaveActionRunner(ActionRunner):
     def notifyAllRead(self, goodSuites):
@@ -176,9 +181,9 @@ class SlaveActionRunner(ActionRunner):
         # Need to add one if we haven't found any tests though
         if len(goodSuites) == 0:
             ActionRunner.notifyAllRead(self, goodSuites)
-        
+
     def notifyRerun(self, *args):
-        pass # don't rerun directly in the slave, tell the master and give it a chance to send the job elsewhere
+        pass  # don't rerun directly in the slave, tell the master and give it a chance to send the job elsewhere
 
     def notifyNoMoreExtraTests(self):
         self.diag.info("No more extra tests, adding terminator")
@@ -193,7 +198,7 @@ class FindExecutionHostsInSlave(FindExecutionHosts):
 class SlaveMachineInfoFinder(MachineInfoFinder):
     def __init__(self):
         self.queueMachineInfo = None
-        
+
     def findPerformanceMachines(self, test, fileStem):
         perfMachines = []
         resources = test.getCompositeConfigValue("performance_test_resource", fileStem)
@@ -232,4 +237,4 @@ class SlaveMachineInfoFinder(MachineInfoFinder):
         jobs = []
         for user, jobId, jobName in jobsFromQueue:
             jobs.append("Also on " + machine + " : " + user + "'s job " + jobId + " '" + jobName + "'")
-        return jobs    
+        return jobs
